/*-----------------------------------------------------------------------------+
 | TrimClip                                                                    |
 | (c) 2005 Steven Don                                                         |
 | http://www.shdon.com                                                        |
 +-----------------------------------------------------------------------------+
 | Utility to sit in the taskbar notification area and monitor the clipboard.  |
 | If it detects text being copied into the clipboard that has trailing        |
 | whitespace, that whitespace is trimmed.                                     |
 +-----------------------------------------------------------------------------+
 | License: You may use all or any part this code in any way you see fit, with |
 |          the exception of harassing the author or any other people (so I    |
 |          guess that means SCO can't use it). Crediting the original author  |
 |          is appreciated, but not required.                                  |
 +-----------------------------------------------------------------------------*/

#include <windows.h>
#include <afxres.h>

#include "resource.h"

#define APPNAME "TrimClip"
#define WM_NOTIFYICON WM_APP

HWND ghWndNext;
HMENU hMenu;
bool enabled = true, updating;

//Add the icon to the taskbar
void tbAddIcon (HWND hWnd, long ID, HICON hIcon, char *toolTip)
{
	NOTIFYICONDATA notifyIconData = { sizeof (NOTIFYICONDATA) };
	notifyIconData.hWnd = hWnd;
	notifyIconData.uID = ID;
	notifyIconData.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP;
	notifyIconData.uCallbackMessage = WM_NOTIFYICON;
	notifyIconData.hIcon = hIcon;
	if (toolTip) lstrcpyn (notifyIconData.szTip, toolTip, sizeof (notifyIconData.szTip));
	Shell_NotifyIcon (NIM_ADD, &notifyIconData);
}

//Remove the icon from the taskbar
void tbRemoveIcon (HWND hWnd, long ID)
{
	NOTIFYICONDATA notifyIconData = { sizeof (NOTIFYICONDATA) };
	notifyIconData.cbSize = sizeof (NOTIFYICONDATA);
	notifyIconData.hWnd = hWnd;
	notifyIconData.uID = ID;
	Shell_NotifyIcon (NIM_DELETE, &notifyIconData);
}

//Check the clipboard for text
void HandleClipboard (HWND hWnd)
{
	if (OpenClipboard (hWnd))
	{
		HGLOBAL hold = GetClipboardData (CF_TEXT);
		HGLOBAL hnew = NULL;
		if (hold)
		{
			char *str = (char *)GlobalLock (hold);
			if (str)
			{
				//Trim any spaces
				char *end = str + strlen (str);
				char *cur = end;
				while (cur > str)
				{
					if (isspace (cur [-1]))
						--cur;
					else
						break;
				}

				//Spaces trimmed?
				if (cur != end && cur > str)
				{
					//Create copy
					int len = cur - str;
					hnew = GlobalAlloc (GHND | GMEM_DDESHARE, len + 1);
					if (hnew)
					{
						char *buf = (char *)GlobalLock (hnew);
						memcpy (buf, str, len);
						buf [len] = 0;
						GlobalUnlock (hnew);
					}
				}
			}
			GlobalUnlock (hold);

			//Copy new text to clipboard
			if (hnew)
			{
				EmptyClipboard ();
				SetClipboardData (CF_TEXT, hnew);
			}
		}
		CloseClipboard ();
	}
}

//Window procedure
LRESULT CALLBACK WinProc (HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam)
{
	switch (Msg)
	{
	case WM_CHANGECBCHAIN :
		//A window is removing itself from the clipboard chain
		if ((HWND)wParam == ghWndNext)
			//Adjust the chain
			ghWndNext = (HWND)lParam;
		else if (ghWndNext)
			//Pass on to other handlers
			SendMessage (ghWndNext, Msg, wParam, lParam);
		return 0;

	case WM_CREATE :
		//Add our icon to the taskbar notification area
		tbAddIcon (hWnd, WM_NOTIFYICON, (HICON)LoadImage (GetModuleHandle (NULL), MAKEINTRESOURCE (IDI_MAINICON), IMAGE_ICON, 16, 16, 0), "TrimClip");
		//Maintain the clipboard chain
		ghWndNext = SetClipboardViewer (hWnd);
		break;

	case WM_DESTROY :
		//Remove our icon from the taskbar notification area
		tbRemoveIcon (hWnd, WM_NOTIFYICON);
		//Remove ourselves from the clipboard chain
		ChangeClipboardChain (hWnd, ghWndNext);
		//Shut down the program
		PostQuitMessage (0);
		break;

	case WM_DRAWCLIPBOARD :
		//Handle text only
		if (enabled && !updating && IsClipboardFormatAvailable (CF_TEXT))
		{
			updating = true;
			HandleClipboard (hWnd);
			updating = false;
		}
		//Pass on to next handler
		SendMessage (ghWndNext, Msg, wParam, lParam);
		return 0;

	case WM_NOTIFYICON :
		//Out taskbar icon was clicked
		if (lParam == WM_RBUTTONDOWN)
		{
			//Get the cursor position
			POINT p;
			GetCursorPos (&p);

			//Show the menu
			int result = TrackPopupMenuEx (hMenu, TPM_VERTICAL | TPM_RETURNCMD, p.x, p.y, hWnd, 0);

			switch (result)
			{
			case ID_ENABLED :
				//Toggle usage flag
				enabled ^= true;
				CheckMenuItem (hMenu, ID_ENABLED, MF_BYCOMMAND | (enabled ? MF_CHECKED : MF_UNCHECKED));
				break;
			case ID_APP_EXIT :
				//Close the program
				DestroyWindow (hWnd);
				break;
			}
		}
		return 0;
	}

	return DefWindowProc (hWnd, Msg, wParam, lParam);
}

//Main procedure
int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int nCmdShow)
{
	//Register window class, as standard as can be
	WNDCLASS wc = { 0 };
	wc.lpfnWndProc = WinProc;
	wc.hInstance = hInstance;
	wc.lpszClassName = APPNAME;
	RegisterClass (&wc);

	//Initialise handles
	hMenu = GetSubMenu (LoadMenu (hInstance, MAKEINTRESOURCE (IDR_MENU1)), 0);
	CreateWindowEx (0, APPNAME, 0, 0, 0, 0, 0, 0, 0, 0, hInstance, 0);

	//Start message loop
	MSG msg;
	while (GetMessage (&msg, NULL, 0, 0))
	{
		if (msg.message == WM_QUIT) break;
		TranslateMessage (&msg);
		DispatchMessage (&msg);
	}

	//Clean up
	UnregisterClass (APPNAME, hInstance);

	return 0;
}